;============================================
; Resizing Test Bed
;
; Tests resizing objects (child windows)
; using owner-drawn resizing handles
;
; LESSONS LEARNED IN THIS CODE:
;-------------------------------
; o Loading cursors, setting in child windows
; o Moving multiple child windows as a unit
; o Using pop-up ("shortcut") menus, including
;   using ClientToScreen() and ScreenToClient()
;   to convert between client <--> screen coords.
; o Creating child-window controls "on the fly"
; o Mouse capture, window moving and coordination
;   between parent & child window procs
;
; UNSOLVED PROBLEMS:
;--------------------
; o Static text doesn't show in child window
; o (minor) Control disappears from child window
;   when it's being dragged
;============================================


.386
.model flat, stdcall
option casemap :none

.nolist
	include \masm32\include\windows.inc
	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc
	include \masm32\include\comctl32.inc

	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib
	includelib \masm32\lib\Comctl32.lib
	includelib \masm32\lib\comdlg32.lib
	includelib \masm32\lib\shell32.lib
	includelib \masm32\lib\oleaut32.lib
	includelib \masm32\lib\ole32.lib
	includelib \masm32\lib\msvcrt.lib
.list


;============================================
; Defines, macros, prototypes, etc.
;============================================

WinMain			PROTO :DWORD
MoveHandleWindows	PROTO x0c:DWORD, y0c:DWORD
ResizeChild		PROTO x:DWORD, y:DWORD


$parent			EQU 1
$child			EQU 2

$handleInfo		STRUCT
  ID			DD ?
  X			DD ?
  Y			DD ?
  flags			DD ?
  handle		HWND ?
  cursor		HCURSOR ?
$handleInfo		ENDS

$mainWindowWidth	EQU 700
$mainWindowHeight	EQU 300

$childX			EQU 100
$childY			EQU 100
$Wc			EQU 200
$Hc			EQU 40
$childMinW		EQU 50
$childMinH		EQU 20

$Wh			EQU 6
$Hh			EQU 6

$Xc			EQU $childX
$Yc			EQU $childY

$chkID			EQU 2020
$chkX			EQU 20
$chkY			EQU 10
$chkWidth		EQU 50
$chkHeight		EQU 20

$radioID		EQU 2030
$radioX			EQU 20
$radioY			EQU 10
$radioWidth		EQU 50
$radioHeight		EQU 20

$editID			EQU 2031
$editX			EQU 20
$editY			EQU 10
$editWidth		EQU 50
$editHeight		EQU 20

$STID			EQU 2032
$STX			EQU 20
$STY			EQU 10
$STWidth		EQU 80
$STHeight		EQU 20

$SBID			EQU 2040
$SBX			EQU 0
$SBY			EQU $mainWindowHeight - $SBHeight
$SBWidth		EQU $mainWindowWidth
$SBHeight		EQU 40
$numStatusParts		EQU 5

;===== Window styles: =====
$mainWinAttrs		EQU WS_OVERLAPPEDWINDOW OR WS_CLIPCHILDREN
$childWinAttrs		EQU WS_CHILD OR WS_VISIBLE OR WS_CLIPSIBLINGS
$handleWinAttrs		EQU WS_CHILD OR WS_BORDER OR WS_VISIBLE OR WS_CLIPSIBLINGS
$SB_attrs		EQU WS_CHILD OR WS_VISIBLE OR WS_CLIPSIBLINGS

;===== Window background colors: =====
$bkRED			EQU 254
$bkGRN			EQU 243
$bkBLUE			EQU 199

$whtRED			EQU 255
$whtGRN			EQU 255
$whtBLUE		EQU 255

$BkColor		EQU $bkRED OR ($bkGRN SHL 8) OR ($bkBLUE SHL 16)
$white			EQU $whtRED OR ($whtGRN SHL 8) OR ($whtBLUE SHL 16)

$WM_MOVECHILD		EQU WM_APP + 1

;===== Move handle flag bits: =====
$Vconstrain		EQU 100H
$Hconstrain		EQU 200H
$dX0		EQU 400H
$dY0		EQU 800H
$dXneg		EQU 1000H
$dXpos			EQU 2000H
$dYneg		EQU 4000H
$dYpos			EQU 8000H
$dW		EQU 10000H
$dH		EQU 20000H

$bitA			EQU 80H
$bitB			EQU 40H
$bitC			EQU 20H
$bitD			EQU 10H
$bitE			EQU 8H
$bitF			EQU 4H
$bitG			EQU 2H
$bitH			EQU 1H

;============================================
; DATA
;============================================
.data

InstanceHandle		HWND ?
MainWinHandle		HWND ?
ChildWinHandle		HWND ?
ChkHandle		HWND ?
RadioHandle		HWND ?
EditHandle		HWND ?
STHandle		HWND ?
StatusHandle		HWND ?

ControlHandle		HWND NULL

HandleWinHandle		HWND ?
HandlePtr		DD ?
StartResizePosX		DD ?
StartResizePosY		DD ?

ShortcutMenuHandle	HMENU ?

HandleInfo	LABEL DWORD
  $handleInfo <2000, $Xc - $Wh, $Yc - $Hh,			11111110B OR $dX0 OR $dY0 OR $dW OR $dH OR $dXneg OR $dYneg>
  $handleInfo <2001, $Xc + ($Wc / 2) + ($Wh / 2), $Yc - $Hh,	11111000B OR $Vconstrain OR $dY0 OR $dH OR $dYneg>
  $handleInfo <2002, $Xc + $Wc, $Yc - $Hh,			11111011B OR $dY0 OR $dH OR $dW OR $dXpos OR $dYneg>
  $handleInfo <2003, $Xc - $Wh, $Yc + ($Hc / 2) - ($Hh / 2),	11010110B OR $Hconstrain OR $dX0 or $dW OR $dXneg>
  $handleInfo <2004, $Xc + $Wc, $Yc + ($Hc / 2) - ($Hh / 2),	01101011B OR $Hconstrain OR $dW OR $dXpos>
  $handleInfo <2005, $Xc - $Wh, $Yc + $Hc,			11011111B OR $dX0 OR $dW OR $dH OR $dXneg OR $dYpos>
  $handleInfo <2006, $Xc + ($Wc / 2) + ($Wh / 2), $Yc + $Hc,	00011111B OR $Vconstrain OR $dH OR $dYpos>
  $handleInfo <2007, $Xc + $Wc, $Yc + $Hc,			01111111B OR $dW OR $dH OR $dXpos OR $dYpos>

MM_Xoffset		DD ?
MM_Yoffset		DD ?
SBheight		DD ?
TBheight		DD ?
ChildWidth		DD $Wc
ChildHeight		DD $Hc
ResizeFlag		DD ?

StatusParts		DD 100, 200, 300, 400, -1

PrevRect		RECT <>

;===== Word-size items: =====
MovingChild		DW FALSE
ResizingChild		DW FALSE

;===== Byte-size items: =====
MainClassName		DB "Resizing", 0
ChildClassName		DB "static", 0
BtnClassName		DB "button", 0
EditClassName		DB "edit", 0
StaticClassName		DB "static"
NullString		DB 0

MainDisplayName		DB "Resizing Handle Test", 0
ChildText		DB "Child", 0
HandleText		DB "H", 0
ChkText			DB "Fur:", 0
RadioText		DB "FM", 0
StaticText		DB "Some text", 0

PosOutFmt		DB "Position: %d x %d", 0
SizeOutFmt		DB "Size: %d x %d", 0

NumOutText		DB "###"
  NumOutArea		DB "#", 0

Menu_CheckboxText	DB "Checkbox", 0
Menu_RadioText		DB "Radio button", 0
Menu_EditText		DB "Edit box", 0
Menu_StaticText		DB "Static text", 0

heremsg			DB "Here!", 0
filecreatefailmsg	DB "CreateFile() FAILED!", 0
resizingmsg		DB "Resizing", 0
noresizingmsg		DB "          ", 0

dummy			dd ?


;============================================
; CODE
;============================================
.code


start:
	INVOKE	GetModuleHandle, NULL
	MOV	InstanceHandle, EAX

	INVOKE	WinMain, InstanceHandle
	INVOKE	ExitProcess, EAX


;====================================================================
; Mainline proc
;====================================================================

WinMain	PROC 	hInst:DWORD
	LOCAL	wc:WNDCLASSEX, msg:MSG, brush:HBRUSH, wX:DWORD, wY:DWORD
	LOCAL	gpRect:RECT


; Create  brush to set background color:
	INVOKE	CreateSolidBrush, $BkColor
	MOV	brush, EAX

; Register class for parent window:
	MOV	wc.cbSize, SIZEOF WNDCLASSEX
	MOV	wc.style,  CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
	MOV	wc.lpfnWndProc, OFFSET MainWindowProc
	MOV	wc.cbClsExtra, NULL
	MOV	wc.cbWndExtra, NULL
	MOV	EAX, hInst
	MOV	wc.hInstance, EAX
	MOV	EAX, brush
	MOV	wc.hbrBackground, EAX
	MOV	wc.lpszMenuName,  NULL
	MOV	wc.lpszClassName, OFFSET MainClassName
	MOV	wc.hIcon, NULL
	INVOKE	LoadCursor, NULL, IDC_ARROW
	MOV	wc.hCursor, EAX
	MOV	wc.hIconSm, 0
	INVOKE	RegisterClassEx, ADDR wc

; Register class for child windows:
	MOV	wc.cbSize, SIZEOF WNDCLASSEX
	MOV	wc.style,  CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
	MOV	wc.lpfnWndProc, OFFSET ChildWindowProc
	MOV	wc.cbClsExtra, 0
	MOV	wc.cbWndExtra, 0
	MOV	EAX, hInst
	MOV	wc.hInstance, EAX
	MOV	wc.hbrBackground, COLOR_BTNFACE + 1
	MOV	wc.lpszMenuName,  NULL
	MOV	wc.lpszClassName, OFFSET ChildClassName
	MOV	wc.hIcon, NULL
	INVOKE	LoadCursor, NULL, IDC_ARROW
	MOV	wc.hCursor, EAX
	MOV	wc.hIconSm, 0
	INVOKE	RegisterClassEx, ADDR wc

	INVOKE	GetSystemMetrics, SM_CXSCREEN
	MOV	EDX, $mainWindowWidth
	CALL	CenterDim
	MOV	wX, EAX
	INVOKE	GetSystemMetrics, SM_CYSCREEN
	MOV	EDX, $mainWindowHeight
	CALL	CenterDim
	MOV	wY, EAX

; Create our main window:
	INVOKE	CreateWindowEx, WS_EX_OVERLAPPEDWINDOW, ADDR MainClassName, ADDR MainDisplayName,
		$mainWinAttrs, wX, wY, $mainWindowWidth, $mainWindowHeight,
		NULL, NULL, hInst, NULL
	MOV	MainWinHandle, EAX
	INVOKE	ShowWindow, EAX, SW_SHOWNORMAL

; Create the status bar:
	INVOKE	CreateStatusWindow, $SB_attrs, OFFSET NullString, MainWinHandle, $SBID
	MOV	StatusHandle, EAX
	MOV	EDX, EAX
; Get the height of the status bar:
	INVOKE	GetWindowRect, EDX, ADDR gpRect
	MOV	EAX, gpRect.bottom
	SUB	EAX, gpRect.top
	MOV	SBheight, EAX
; Divide status bar into parts:
	INVOKE	SendMessage, StatusHandle, SB_SETPARTS, $numStatusParts, ADDR StatusParts

; Load resizing-handle cursors:
	CALL	LoadArrowCursors

; Create child window:
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR ChildClassName, NULL,
		$childWinAttrs, $childX, $childY, $Wc, $Hc,
		MainWinHandle, 1999, hInst, NULL
	MOV	ChildWinHandle, EAX
	INVOKE	UpdateWindow, EAX

; Create handle windows:
	PUSH	EBX
	LEA	EBX, HandleInfo				;Point to handle info structure array
	MOV	ECX, 8
chw10:	PUSH	ECX
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR ChildClassName, NULL, $handleWinAttrs,
     		[EBX + $handleInfo.X], [EBX + $handleInfo.Y], $Wh, $Hh,
		MainWinHandle, [EBX + $handleInfo.ID], InstanceHandle, 0
	MOV	[EBX + $handleInfo.handle], EAX		;Store handle.
	INVOKE	UpdateWindow, EAX			;Force child wndproc to paint window.
	ADD	EBX, SIZEOF $handleInfo
	POP	ECX
	LOOP	chw10
chw99:	POP	EBX


;============= Message loop ===================
msgloop:
	INVOKE	GetMessage, ADDR msg, NULL, 0, 0
	OR	EAX, EAX				;EAX = 0 = exit
	JZ	exit
	INVOKE	TranslateMessage, ADDR msg
	INVOKE	DispatchMessage, ADDR msg
	JMP	msgloop

exit:
	MOV	EAX, msg.wParam
	RET

WinMain		ENDP




;====================================================================
; Main Window Proc
;====================================================================

MainWindowProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
	LOCAL	pt:POINT, x0c:DWORD, y0c:DWORD, gpRect:RECT, hMenu:HMENU
	local	buffer[40]:BYTE, deltaX:DWORD, deltaY:DWORD, tempWidth:DWORD, tempHeight:DWORD

	MOV	EAX, uMsg
	cmp	EAX, WM_MOUSEMOVE
	je	domousemove
	CMP	EAX, WM_LBUTTONUP
	JE	lbuttonup
	CMP	EAX, WM_RBUTTONDOWN
	JE	rbuttondown
	CMP	EAX, WM_COMMAND
	JE	dowincmd
;	CMP	EAX, WM_CREATE
;	JE	docreate
	CMP	EAX, WM_CLOSE
	JE	doclose

dodefault:
; use DefWindowProc for all other messages:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

domousemove:
; Get current mouse position (from lParam), put in x0c, y0c:
	MOV	EAX, lParam			;Get new X & Y pos.
	MOV	EDX, EAX			;Make a copy
	AND	EAX, 0FFFFH			;Mask off high word
	CWDE
	MOV	x0c, EAX
	SHR	EDX, 16				;High word--> low word
	MOV	EAX, EDX
	CWDE
	MOV	y0c, EAX
	CMP	MovingChild, TRUE
	JE	do_move
	CMP	ResizingChild, TRUE
	JNE	dodefault

; Resize child:
	MOV	EDX, HandlePtr
	MOV	EAX, [EDX + $handleInfo.flags]
	MOV	ResizeFlag, EAX
	INVOKE	ResizeChild, x0c, y0c
	INVOKE	wsprintf, ADDR buffer, ADDR SizeOutFmt, ChildWidth, ChildHeight
	INVOKE  SendMessage, StatusHandle, SB_SETTEXT, 3, ADDR buffer
	XOR	EAX, EAX
	RET

do_move:
	MOV	EAX, MM_Xoffset
	SUB	x0c, EAX
	MOV	EAX, MM_Yoffset
	SUB	y0c, EAX
	INVOKE	SetWindowPos, ChildWinHandle, HWND_BOTTOM, x0c, y0c, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, ChildWinHandle, NULL, TRUE
	INVOKE	MoveHandleWindows, x0c, y0c
	INVOKE	wsprintf, ADDR buffer, ADDR PosOutFmt, x0c, y0c
	INVOKE  SendMessage, StatusHandle, SB_SETTEXT, 2, ADDR buffer
	XOR	EAX, EAX
	RET

lbuttonup:
	CMP	MovingChild, TRUE		;Are we watching the mouse?
	JE	release_move
	CMP	ResizingChild, TRUE
	JNE	dodefault
	MOV	ResizingChild, FALSE
	call	putnoresizing
	JMP	SHORT do_release
release_move:
	MOV	MovingChild, FALSE		;Yep, clear flag.
do_release:
	INVOKE	ReleaseCapture
 	INVOKE	ClipCursor, ADDR PrevRect	;Un-clip to our window.
	JMP	mwp_commonexit

rbuttondown:
	CALL	CreatePopupMenu			;Create empty shortcut menu
	MOV	hMenu, EAX
	INVOKE	AppendMenu, hMenu, MF_STRING, 2222, OFFSET Menu_CheckboxText
	INVOKE	AppendMenu, hMenu, MF_STRING, 2223, OFFSET Menu_RadioText
	INVOKE	AppendMenu, hMenu, MF_STRING, 2225, OFFSET Menu_StaticText
	INVOKE	AppendMenu, hMenu, MF_STRING, 2224, OFFSET Menu_EditText
	MOV	EAX, lParam
	MOV	EDX, EAX
	AND	EAX, 0FFFFH
	CWDE
	MOV	pt.x, EAX
	SHR	EDX, 16
	CWDE
	MOV	pt.y, EDX
	INVOKE	ClientToScreen, hWin, ADDR pt	;Where is our client in the "real world"?
	INVOKE	TrackPopupMenuEx, hMenu,
		TPM_LEFTALIGN OR TPM_TOPALIGN OR TPM_RETURNCMD OR TPM_RIGHTBUTTON OR TPM_VERNEGANIMATION OR TPM_VERTICAL,
		pt.x, pt.y, hWin, NULL

	CMP	EAX, 2222
	JE	makeCheckbox
	CMP	EAX, 2223
	JE	makeRadio
	CMP	EAX, 2224
	JE	makeEdit
	CMP	EAX, 2225
	JE	makeStaticText

	XOR	EAX, EAX
	RET

dowincmd:

docreate:
mwp_commonexit:
	XOR	EAX, EAX
	RET

doclose:
	INVOKE	PostQuitMessage, NULL
	MOV	EAX, TRUE
	RET

makeCheckbox:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR BtnClassName, ADDR ChkText,
		WS_TABSTOP OR WS_VISIBLE OR WS_CHILD OR BS_AUTOCHECKBOX, $chkX, $chkY,
		$chkWidth, $chkHeight, ChildWinHandle, $chkID, InstanceHandle, NULL
	MOV	ChkHandle, EAX
	MOV	ControlHandle, EAX
	INVOKE	CheckDlgButton, ChildWinHandle, $chkID, BST_CHECKED	;Set button state to checked.
	INVOKE	UpdateWindow, ChkHandle
	JMP	mwp_commonexit

makeRadio:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR BtnClassName, ADDR RadioText,
		WS_TABSTOP OR WS_VISIBLE OR WS_CHILD OR BS_AUTORADIOBUTTON, $radioX, $radioY,
		$radioWidth, $radioHeight, ChildWinHandle, $radioID, InstanceHandle, NULL
	MOV	RadioHandle, EAX
	MOV	ControlHandle, EAX
	INVOKE	CheckDlgButton, ChildWinHandle, $radioID, BST_CHECKED	;Set button state to checked.
	INVOKE	UpdateWindow, RadioHandle
	JMP	mwp_commonexit

makeEdit:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR EditClassName, NULL,
		SS_LEFT OR WS_VISIBLE OR WS_CHILD OR WS_BORDER, $editX, $editY,
		$editWidth, $editHeight, ChildWinHandle, $editID, InstanceHandle, NULL
	MOV	EditHandle, EAX
	MOV	ControlHandle, EAX
	INVOKE	UpdateWindow, EAX
	JMP	mwp_commonexit

makeStaticText:
	CALL	DestroyExistingControl
	INVOKE	CreateWindowEx, WS_EX_LEFT, ADDR StaticClassName, ADDR StaticText,
		SS_LEFT OR WS_VISIBLE OR WS_CHILD, $STX, $STY,
		$STWidth, $STHeight, ChildWinHandle, $STID, InstanceHandle, NULL
	MOV	STHandle, EAX
	MOV	ControlHandle, EAX
	INVOKE	UpdateWindow, EAX
	JMP	mwp_commonexit

MainWindowProc	ENDP


;====================================================================
; Child Window Proc
;====================================================================

ChildWindowProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD
	LOCAL	gpRect:RECT, x0c:DWORD, y0c:DWORD, pt:POINT, x1:DWORD, y1:DWORD
	LOCAL	buffer[100]:BYTE
	local	offsetX:DWORD, offsetY:DWORD

	MOV	EAX, uMsg
;	CMP	EAX, WM_MOUSEMOVE
;	JE	domousemove
	CMP	EAX, WM_SETCURSOR
	JE	setcursor
	CMP	EAX, WM_LBUTTONDOWN
	JE	lbuttondown
	CMP	EAX, WM_LBUTTONUP
	JE	lbuttonup
	CMP	EAX, WM_RBUTTONDOWN
	JE	rbuttondown

dodefault:
; use DefWindowProc for all other messages:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

setcursor:
	MOV	EAX, hWin
	CALL	FindHandleWindow		;See if handle matches any handle window's:
	OR	EAX, EAX			;Any match?
	JZ	dodefault			;Nope, do nothing.
	INVOKE	SetCursor, [EDX + $handleInfo.cursor]
	XOR	EAX, EAX
	RET

lbuttondown:
	CMP	MovingChild, TRUE
	JE	dodefault
	CMP	ResizingChild, TRUE
	JE	dodefault

; Get current mouse position, save as initial move/resize offset:
	MOV	EAX, lParam			;Get new X & Y pos.
	MOV	EDX, EAX			;Make a copy for later
	AND	EAX, 0FFFFH			;Mask off high word
	CWDE					;Sign-extend to DWORD
	MOV	MM_Xoffset, EAX
	MOV	EAX, EDX			;Get back copy
	SHR	EAX, 16				;High word--> low word
	CWDE
	MOV	MM_Yoffset, EAX

; See which child window (main child or handles) generated the message:
	MOV	EAX, hWin
	CMP	EAX, ChildWinHandle
	JE	lbutton_child
	CALL	FindHandleWindow
	OR	EAX, EAX
	JZ	dodefault			;No match, do the default thang.
	MOV	HandleWinHandle, EAX
	MOV	HandlePtr, EDX
	MOV	ResizingChild, TRUE
	call	putresizing
; Compute starting position from child's POV:
	MOV	EAX, MM_Xoffset
	MOV	pt.x, EAX
	MOV	EAX, MM_Yoffset
	MOV	pt.y, EAX
	INVOKE	ClientToScreen, hWin, ADDR pt
	INVOKE	ScreenToClient, MainWinHandle, ADDR pt
; Calculate pos. from parent's POV:
	MOV	EAX, pt.x
	MOV	StartResizePosX, EAX
	MOV	EAX, pt.y
	MOV	StartResizePosY, EAX
	JMP	SHORT cwp_capture

lbutton_child:
; Capture that rodent!
	MOV	MovingChild, TRUE
cwp_capture:
	INVOKE	SetCapture, MainWinHandle
	INVOKE	GetClipCursor, ADDR PrevRect	;Get current clip rectangle
	INVOKE	GetWindowRect, MainWinHandle, ADDR gpRect
	MOV	EAX, SBheight
	SUB	gpRect.bottom, EAX		;Clip out status bar.
 	INVOKE	ClipCursor, ADDR gpRect
	JMP	SHORT cwp_commonexit

lbuttonup:
	CMP	MovingChild, TRUE		;Are we watching the mouse?
	JE	release_move
	CMP	ResizingChild, TRUE
	JNE	dodefault
	MOV	ResizingChild, FALSE
	call	putnoresizing
	JMP	SHORT do_release
release_move:
	MOV	MovingChild, FALSE		;Yep, clear flag.
do_release:
	INVOKE	ReleaseCapture
 	INVOKE	ClipCursor, ADDR PrevRect	;Un-clip to our window.

cwp_commonexit:
	XOR	EAX, EAX
	RET

rbuttondown:
; Post this message to parent window so it can process it:

	MOV	EAX, lParam
	MOV	EDX, EAX
	AND	EAX, 0FFFFH
	MOV	pt.x, EAX
	SHR	EDX, 16
	MOV	pt.y, EDX
	INVOKE	ClientToScreen, hWin, ADDR pt	;Where is our client in the "real world"?
	INVOKE	ScreenToClient, MainWinHandle, ADDR pt
	MOV	EAX, pt.y
	SHL	EAX, 16
	MOV	AX, WORD PTR pt.x
	INVOKE	PostMessage, MainWinHandle, WM_RBUTTONDOWN, wParam, EAX
	JMP	dodefault


ChildWindowProc	ENDP

;====================================================================
; ResizeChild() (x, y)
;
; Takes care of resizing child window and moving resizing handles.
;====================================================================

ResizeChild	PROC x:DWORD, y:DWORD
		LOCAL	deltaX:DWORD, deltaY:DWORD, pt:POINT

; Compute delta-X and delta-Y of move:
	MOV	EAX, x
	SUB	EAX, StartResizePosX
	MOV	deltaX, EAX
	MOV	EAX, y
	SUB	EAX, StartResizePosY
	MOV	deltaY, EAX

; Check that resulting new child window size is above minimum:
	MOV	EAX, deltaX
	TEST	ResizeFlag, $dXneg
	JZ	rcw3
	NEG	EAX
rcw3:	MOV	EDX, ChildWidth
	ADD	EDX, EAX
	CMP	EDX, $childMinW
	JBE	rcw99				;No good, too small.
	MOV	EAX, deltaY
	TEST	ResizeFlag, $dYneg
	JZ	rcw6
	NEG	EAX
rcw6:	MOV	EDX, ChildHeight
	ADD	EDX, EAX
	CMP	EDX, $childMinH
	JBE	rcw99
	MOV	EAX, deltaX
	ADD	StartResizePosX, EAX
	MOV	EAX, deltaY
	ADD	StartResizePosY, EAX

; Get current (0,0) of child window:
	XOR	EAX, EAX
	MOV	pt.x, EAX
	MOV	pt.y, EAX 
	INVOKE	ClientToScreen, ChildWinHandle, ADDR pt
	INVOKE	ScreenToClient, MainWinHandle, ADDR pt

; Determine whether & how to change origin of child:
	MOV	EAX, deltaX
	MOV	EDX, deltaY
	TEST	ResizeFlag, $dX0
	JZ	rcw10
	ADD	pt.x, EAX
rcw10:	TEST	ResizeFlag, $dY0
	JZ	rcw20
	ADD	pt.y, EDX


; Determine how to change child's size:
rcw20:	TEST	ResizeFlag, $dXneg
	JZ	rcw22
	NEG	EAX
rcw22:	TEST	ResizeFlag, $dYneg
	JZ	rcw24
	NEG	EDX
rcw24:	TEST	ResizeFlag, $dW
	JZ	rcw30
	ADD	ChildWidth, EAX
rcw30:	TEST	ResizeFlag, $dH
	JZ	rcw40
	ADD	ChildHeight, EDX
rcw40:	
	INVOKE	SetWindowPos, ChildWinHandle, HWND_BOTTOM, pt.x, pt.y, ChildWidth, ChildHeight, SWP_SHOWWINDOW
	INVOKE	UpdateWindow, ChildWinHandle
	INVOKE	MoveHandleWindows, pt.x, pt.y
rcw99:	RET
	
ResizeChild	ENDP



putresizing	proc
        INVOKE  SendMessage, StatusHandle, SB_SETTEXT, 4, addr resizingmsg
	ret
putresizing	endp

putnoresizing	proc
        INVOKE  SendMessage, StatusHandle, SB_SETTEXT, 4, addr noresizingmsg
	ret
putnoresizing	endp


DestroyExistingControl	PROC

	CMP	ControlHandle, NULL		;Has a control been created?
	JE	dec99				;No, do nothing.
	INVOKE	DestroyWindow, ControlHandle
	MOV	ControlHandle, NULL
dec99:	RET

DestroyExistingControl	ENDP


;====================================================================
; MoveHandleWindows()
;
; Move sizing-handle windows relative to new (0,0) of the child
; window they surround.
;====================================================================

MoveHandleWindows	PROC x0c:DWORD, y0c:DWORD

	PUSH	EBX
	LEA	EBX, HandleInfo
; 1:
	MOV	EAX, x0c
	SUB	EAX, $Wh
	MOV	EDX, y0c
	SUB	EDX, $Hh
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 2: X = x0c + ($Wc / 2) + ($Wh / 2); Y = y0c - $Hh
	MOV	EAX, ChildWidth
	SHR	EAX, 1
	ADD	EAX, $Wh SHR 1
	ADD	EAX, x0c
	MOV	EDX, y0c
	SUB	EDX, $Hh
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 3: X = x0c + $Wc; Y = y0c - $Hh
	MOV	EAX, x0c
	ADD	EAX, ChildWidth
	MOV	EDX, y0c
	SUB	EDX, $Hh
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 4: X = x0c - $Wh; Y = y0c + ($Hc / 2) - ($Hh / 2)
	MOV	EAX, x0c
	SUB	EAX, $Wh
	MOV	EDX, ChildHeight
	SHR	EDX, 1
	SUB	EDX, $Hh SHR 1
	ADD	EDX, y0c
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 5: X = x0c + $Wc; Y = y0c + ($Hc / 2) - ($Hh / 2)
	MOV	EAX, x0c
	ADD	EAX, ChildWidth
	MOV	EDX, ChildHeight
	SHR	EDX, 1
	SUB	EDX, $Hh SHR 1
	ADD	EDX, y0c
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 6: X = x0c - $Wh; Y = y0c + $Hc
	MOV	EAX, x0c
	SUB	EAX, $Wh
	MOV	EDX, y0c
	ADD	EDX, ChildHeight
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 7: X = x0c + ($Wc / 2) + ($Wh / 2); Y = y0c + $Hc
	MOV	EAX, ChildWidth
	SHR	EAX, 1
	ADD	EAX, x0c
	ADD	EAX, ($Wh SHR 1)
	MOV	EDX, y0c
	ADD	EDX, ChildHeight
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	ADD	EBX, SIZEOF $handleInfo
; 8: X = x0c + $Wc; Y = y0c + $Hc
	MOV	EAX, x0c
	ADD	EAX, ChildWidth
	MOV	EDX, y0c
	ADD	EDX, ChildHeight
	INVOKE	SetWindowPos, [EBX + $handleInfo.handle], HWND_BOTTOM, EAX, EDX, 0, 0, SWP_NOSIZE OR SWP_SHOWWINDOW
	INVOKE	InvalidateRect, [EBX + $handleInfo.handle], NULL, TRUE
	
	POP	EBX
	RET

MoveHandleWindows	ENDP


LoadArrowCursors	PROC
	LOCAL	cursorNS:HCURSOR, cursorEW:HCURSOR, cursorLdiag:HCURSOR, cursorRdiag:HCURSOR

; Load 4 resizing cursors:
	INVOKE	LoadCursor, NULL, IDC_SIZENS
	MOV	cursorNS, EAX
	INVOKE	LoadCursor, NULL, IDC_SIZEWE
	MOV	cursorEW, EAX
	INVOKE	LoadCursor, NULL, IDC_SIZENWSE
	MOV	cursorLdiag, EAX
	INVOKE	LoadCursor, NULL, IDC_SIZENESW
	MOV	cursorRdiag, EAX

; Store cursor handles in handle window table:
	PUSH	ESI
	PUSH	EBX
	LEA	ESI, HandleInfo
	MOV	EAX, cursorLdiag
	MOV	EBX, cursorEW
	MOV	ECX, cursorRdiag
	MOV	EDX, cursorNS
; 1:
	MOV	[ESI + $handleInfo.cursor], EAX
; 2:
	MOV	[ESI + (SIZEOF $handleInfo) + $handleInfo.cursor], EDX
; 3:
	MOV	[ESI + (SIZEOF $handleInfo * 2) + $handleInfo.cursor], ECX
; 4:
	MOV	[ESI + (SIZEOF $handleInfo * 3) + $handleInfo.cursor], EBX
; 5:
	MOV	[ESI + (SIZEOF $handleInfo * 4) + $handleInfo.cursor], EBX
; 6:
	MOV	[ESI + (SIZEOF $handleInfo * 5) + $handleInfo.cursor], ECX
; 7:
	MOV	[ESI + (SIZEOF $handleInfo * 6) + $handleInfo.cursor], EDX
; 8:
	MOV	[ESI + (SIZEOF $handleInfo * 7) + $handleInfo.cursor], EAX

	POP	EBX
	POP	ESI
	RET

LoadArrowCursors	ENDP


;====================================================================
; FindHandleWindow()
;
; Looks for entry matching a window handle
; If found, returns the handle and the pointer to the handle table entry
;
; On entry,
;   EAX = handle to match
;
; On exit,
;   EAX = handle if found, NULL otherwise
;   EDX--> handle table entry
;====================================================================

FindHandleWindow	PROC

	MOV	ECX, 8
	LEA	EDX, HandleInfo
fhw10:	CMP	EAX, [EDX + $handleInfo.handle]	;Check handle for match.
	JE	fhw44				;Bingo! Return w/cursor.
	ADD	EDX, SIZEOF $handleInfo
	LOOP	fhw10
	XOR	EAX, EAX			;Fell through, no match.
	RET

fhw44:	MOV	EAX, [EDX + $handleInfo.handle]
	RET

FindHandleWindow	ENDP



;====================================================================
; CenterDim
;
; Returns half screen dimension (X or Y) minus half window dimension
;
; On entry,
;	EAX = screen dimension
;	EDX = window dimension
;
; Returns:
;	sdim/2 - wdim/2
;====================================================================

CenterDim	PROC

	SHR	EAX, 1				;divide screen dimension by 2
	SHR	EDX, 1				;divide window dimension by 2
	SUB	EAX, EDX
	RET					;Return w/# in EAX

CenterDim	ENDP


;====================================================================
; B2A4/8
;
; Converts binary value to ASCII numeric digits (4/8 digits).
; Result is leading-zero suppressed.
;
; On entry,
;	EAX = value to convert
;	EDI--> END of location to store digits (4/8)
;
; Returns: (nothing)
;
;====================================================================

B2A4		PROC
; *** Code courtesy of "dedndave" from The MASM Forum  ***

	PUSH	ECX
	MOV	ECX, 4			;4 digits
	JMP	SHORT b2a11

B2A8	LABEL	NEAR			;Alternate entry point (8 digits)
	PUSH	ECX
	MOV	ECX, 8
	JMP	SHORT b2a11

b2a10:	OR	EAX, EAX
        JZ      b2a12			;If # is zero, just put in one '0'.
b2a11:	PUSH	ECX
	MOV	CL, 10
	XOR	EDX, EDX
	DIV	ECX
	OR	DL, '0'			;Binary--> ASCII
	POP	ECX
	MOV	[EDI], DL
	DEC	EDI
	LOOP	b2a10
b2a99:	POP	ECX
	RET

b2a12:	MOV	BYTE PTR [EDI],  ' '
	DEC	EDI
	LOOP	b2a12
	JMP	b2a99

B2A4		ENDP


	END	start



